/* ----------------------------------------------------------------------
 * dumptlb.c
 *
 * Copyright (C) 1994, 1995 by Waldorf Electronics,
 * written by Ralf Baechle and Andreas Busse
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file COPYING in the main directory of this archive for
 * more details.
 * ---------------------------------------------------------------------- */

#include <asm/mipsregs.h>
#include <linux/string.h>

#include "console.h"
#include <arc/arc.h>

static char temp[1024];

#define VALID 0x02ULL		/* valid bit */

void
dump_tlb(void)
{
  int	i, wired, dummy;
  unsigned long long entry_hi, entry_lo0, entry_lo1;

  printf("\r\n\r\nTLB dump:\r\n");
  wired = read_32bit_cp0_register(CP0_WIRED);
  printf("Wired: %d\r\n\r\n", wired);
  
  for(i=0 ; i<48; i++)		/* print all TLB entries */
    {
      write_32bit_cp0_register(CP0_INDEX, i);
      __asm__ __volatile__("nop;tlbr");
      
      entry_hi = read_64bit_cp0_register(CP0_ENTRYHI);
      entry_lo0 = read_64bit_cp0_register(CP0_ENTRYLO0);
      entry_lo1 = read_64bit_cp0_register(CP0_ENTRYLO1);
      
      if ((entry_lo0 && VALID) || (entry_lo1 && VALID) || 1)
	{
	  printf("TLB #%d:\r\n",i);
	  printf("\tEntryHi : %08x:%08x\t",
		(unsigned int)(entry_hi >> 32), (unsigned int) entry_hi);
	  printf("(R=%1x VADR=%02x:%08x ASID=%04x)\r\n",
		 entry_hi >> 62,
		 entry_hi & 0xff00000000ULL,
		 entry_hi & 0xffffe000ULL,
		 entry_hi & 0xffULL);
	  printf("\tEntryLo0: %08x:%08x\t",
		(unsigned int)(entry_lo0 >> 32), (unsigned int) entry_lo0);
	  printf("(PADR=%02x:%08x C=%d D=%d V=%d G=%d)\r\n",
		 (entry_lo0 & 0x3c000000) >> 25,
		 (entry_lo0 & 0x03ffffc0) << 6,
		 (entry_lo0 & 0x00000038) >> 3,
		 (entry_lo0 & 0x00000004) >> 2,
		 (entry_lo0 & 0x00000002) >> 1,
		 (entry_lo0 & 0x00000001));
	  printf("\tEntryLo1: %08x:%08x\t",
		(unsigned int)(entry_lo1 >> 32), (unsigned int) entry_lo1);
	  printf("(PADR=%02x:%08x C=%d D=%d V=%d G=%d)\r\n",
		 (entry_lo1 & 0x3c000000) >> 25,
		 (entry_lo1 & 0x03ffffc0) << 6,
		 (entry_lo1 & 0x00000038) >> 3,
		 (entry_lo1 & 0x00000004) >> 2,
		 (entry_lo1 & 0x00000002) >> 1,
		 (entry_lo1 & 0x00000001));
	  printf("\r\nPageMask: %08x\r\n\r",
		 read_32bit_cp0_register(CP0_PAGEMASK));
	  printf("Press a key for next\r\n");
	  getchar();
	}
    }
}
