/*
 * get_nlist.c -- This file contains the routines for getting a symbol
 *		  table entry when booting the Linux/MIPS kernel.
 *
 * Copyright 1993 by Hamish Macdonald
 *
 * This file is subject to the terms and conditions of the
 * GNU General Public License. See the file COPYING in the
 * main directory of this archive for more details.
 *
 */

#include <arc/arc.h>

#include <sys/fcntl.h>
#include <sys/file.h>

#include <linux/a.out.h>
#include <linux/string.h>

extern long memptr;
extern struct exec kexec;

struct nlist *get_nlist (const char *fname, const char *symname)
{
    int fd;
    struct exec ex;
    struct nlist *nl, *p = NULL, *syms;
    char *strs;
    off_t filesize;
    long strsize, numsyms;

    fd = open (fname, O_RDONLY);
    if (fd == -1)
	return NULL;

    read (fd, &ex, sizeof(ex));
    if (!ex.a_syms) {
	close (fd);
	return NULL;
    }

    syms = (struct nlist *)malloc (ex.a_syms);
    if (!syms) {
	close (fd);
	return NULL;
    }

    lseek (fd, N_SYMOFF(ex), L_SET);
    read (fd, syms, ex.a_syms);
    numsyms = ex.a_syms / sizeof (struct nlist);
    filesize = lseek (fd, 0L, L_XTND);
    strsize = filesize - N_STROFF(ex);
    strs = (char *)malloc (strsize);

    if (!strs) {
	free (syms);
	close (fd);
	return NULL;
    }

    lseek (fd, N_STROFF(ex), L_SET);
    read (fd, strs, strsize);

    for (nl = syms; nl < syms + numsyms; nl++) {
	/*
	 * We accept only extern visible .text, .data and .bss symbols.
	 */
	if (strcmp (symname, strs + nl->n_un.n_strx) == 0
	    && ((nl->n_type == N_TEXT | N_EXT) ||
	        (nl->n_type == N_DATA | N_EXT) ||
	        (nl->n_type == N_BSS  | N_EXT))) {
	    p = (struct nlist *)malloc (sizeof (struct nlist)
					+ strlen(strs + nl->n_un.n_strx) + 1);
	    if (!p)
		break;
	    *p = *nl;
	    p->n_un.n_name = (char *)(p+1);
	    strcpy (p->n_un.n_name, strs + nl->n_un.n_strx);
	}
    }
    free (strs);
    free (syms);
    close (fd);
    return p;
}
